/**
 * Edited version of
 * Lee Stemkoski texture animator for A-Frame
 * by @jor3l
 * v-0.1
 * Edited by Matt Napolillo
 */

if (typeof AFRAME === 'undefined') {
	throw new Error('Component attempted to register before AFRAME was available.');
}
  
AFRAME.registerComponent('texture-animator', {
	schema: {
		htiles: {
			default: 1
        },
        vtiles: {
         	default: 1
        },
        tiles: {
            default: 1
        },
        dur: {
            default: 10
		},
        src: {
            default: ''
        }
    },
  
	init: function() {
		if(this.data.src == null) {
			console.log("Can't run texture-animator on element without source image.");
			return false;
		}
		
		// note: texture passed by reference, will be updated by the update function.
		this.tilesHorizontal = this.data.htiles;
		this.tilesVertical = this.data.vtiles;
		// how many images does this spritesheet contain?
		//  usually equals tilesHoriz * tilesVert, but not necessarily,
		//  if there at blank tiles at the bottom of the spritesheet. 
		
		this.numberOfTiles = this.data.tiles;
		this.texture = new THREE.ImageUtils.loadTexture( this.data.src ); // Gets the texture from the element
		
		// Set image to doubleSide:
		// Grab the mesh / scene.
		this.el.addEventListener('model-loaded', () => {
			const obj = this.el.getObject3D('mesh');
			// Go over the submeshes and modify materials we want.
			obj.traverse(node => {
				if(node.isMesh && node.name.includes("pcisePen_rigLock_Arrow")){
					node.material.map = this.texture;
				} else if(node.isMesh && node.name.includes("pcisePen_rigCap_Arrow")){
					node.material.map = this.texture;
				}
			});
		});
		// this.el.object3DMap.mesh.material.map = this.texture;
		// this.el.object3DMap.mesh.material.side = THREE.DoubleSide;

		this.texture.wrapS = this.texture.wrapT = THREE.RepeatWrapping; 
		this.texture.repeat.set( 1 / this.tilesHorizontal, 1 / this.tilesVertical );

		// how long should each image be displayed?
		this.tileDisplayDuration = this.data.dur;

		// how long has the current image been displayed?
		this.currentDisplayTime = 0;

		// which image is currently being displayed?
		this.currentTile = 0;
	},
	tick: function () {
		this.currentDisplayTime += 1;
		while (this.currentDisplayTime > this.tileDisplayDuration)
		{
			this.currentDisplayTime -= this.tileDisplayDuration;
			this.currentTile++;

			if (this.currentTile == this.numberOfTiles) {
				this.currentTile = 0;
			}

			var currentColumn = this.currentTile % this.tilesHorizontal;
			this.texture.offset.x = currentColumn / this.tilesHorizontal;

			var currentRow = Math.floor( this.currentTile / this.tilesHorizontal );
			this.texture.offset.y = currentRow / this.tilesVertical;
		}
	},
});